#
# Turing -- Ruby implementation of Captcha
#
# Copyright © 2005 Michal Šafránek <wejn@box.cz>
#
# This file is part of http://turing.rubyforge.org/
#
# See turing.rb in lib/ directory for license terms.
#

# == Squaring helper
# Skeleton for Turing::Image::*Squaring (mixin, actually).
module Turing::Image::SquaringHelper # {{{
	# contract method - generate the challenge
	def generate(img, word, bg = nil) # {{{
		if bg.nil?
			possible = Dir[File.join(@options[:bgdir], '*')]
			bg = possible[rand(possible.size)]
		else
			unless FileTest.exists?(bg)
				raise ArgumentError, "Wrong background!"
			end
		end

		img_tmp = GD2::Image.load(File.open(bg, 'r'))

		if img_tmp.width < img.width || img_tmp.height < img.height
			raise "Background has insufficient dimensions"
		end

		img.merge_from(img_tmp, 0, 0, 0, 0, img.width, img.height, 30.percent)

		# XXX: is this equivalent to "img_tmp.destroy" ?
		img_tmp = nil

		fg = GD2::Color[0, 0, 0]

		write_string(img, 'cour.ttf', fg, word, 35)

		raise RuntimeError, "no squaring color selected!" if @squaring_color.nil?
		fg = @squaring_color
		
		img.draw do |canvas|
			canvas.color = fg
			if rand(2).zero?
				delta = word.size > 8 ? 6 : 4
				0.step(img.width, delta) { |x| canvas.line(x, 0, x, img.height) }
				0.step(img.height, delta) { |y| canvas.line(0, y, img.width, y) }
			else
				i = 0
				while i <= img.width
					canvas.line(i, 0, i, img.height)
					i += 3 + rand(4)
				end

				i = 0
				while i <= img.height
					canvas.line(0, i, img.width, i)
					i += 3 + rand(4)
				end
			end
		end
	end # }}}
end # }}}

# vim: set ts=4 sw=4 :
